@@ -738,7 +738,6 @@ list_adt(struct ipset_session *session, struct nlattr *nla[])
 	const struct ipset_data *data = session->data;
 	const struct ipset_type *type;
 	const struct ipset_arg *arg;
-	uint8_t family;
 	int i, found = 0;
 
 	D("enter");
@@ -750,7 +749,6 @@ list_adt(struct ipset_session *session, struct nlattr *nla[])
 
 	if (type == NULL)
 		return MNL_CB_ERROR;
-	family = ipset_data_family(data);
 
 	for (i = IPSET_ATTR_UNSPEC + 1; i <= IPSET_ATTR_ADT_MAX; i++)
 		if (nla[i]) {
@@ -1466,21 +1464,21 @@ rawdata2attr(struct ipset_session *session, struct nlmsghdr *nlh,
 		return 1;
 
 	switch (attr->type) {
-	case MNL_TYPE_NUL_STRING:
-		alen = strlen((const char *)d) + 1;
-		break;
 	case MNL_TYPE_U32: {
 		uint32_t value = htonl(*(const uint32_t *)d);
 
-		d = &value;
-		break;
+		mnl_attr_put(nlh, type | flags, alen, &value);
+		return 0;
 	}
 	case MNL_TYPE_U16: {
 		uint16_t value = htons(*(const uint16_t *)d);
 
-		d = &value;
-		break;
+		mnl_attr_put(nlh, type | flags, alen, &value);
+		return 0;
 	}
+	case MNL_TYPE_NUL_STRING:
+		alen = strlen((const char *)d) + 1;
+		break;
 	default:
 		break;
 	}
